// a comb filter.  If no input, then use whitenoise
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sun Apr 20 19:43:40 GMT-0800 1997
// Last Modified: Sun Apr 20 19:43:43 GMT-0800 1997
// Filename:      .../sig/examples/comb/comb.cc
// Syntax:        C++ 
//

#include "Dependencies.h"

void exitUsage(char *commandName);


int main(int argc, char* argv[]) {
   SoundHeader header;
   long numSamples = 0;
   char* infilename = "";
   char* outfilename= NULL;
   float noiseAmp = 0.0;

   if (argc == 2) {            // no input sound
      header.setHighMono();
      outfilename = argv[1];
      cout << "What amplitude for the noise: [0 .. 1] ";
      cin  >> noiseAmp;
      cout << "How many seconds of noise to use: ";
      float temp;
      cin >> temp;
      numSamples = (long)(temp * 44100);
   } else if (argc == 3) {     // input file specified
      header.setHeader(argv[1]);
      numSamples = header.getSamples() + 10;
      outfilename = argv[2];
      infilename = argv[1];
   } else {
      exitUsage(argv[0]); 
   }

   // Elements:
   SoundFileIn  insound(infilename);
   SoundFileOut outsound(outfilename, header);
   Comb         comb;
   WhiteNoise   noise(noiseAmp);

   // Connections:
   outsound.connect(comb);
   if (strlen(infilename) == 0) {         // use noise
      comb.connect(noise, 0);      
   } else {                               // use input file
      comb.connect(insound, 0);
   }

   
   double fundFreq;
   cout << "What fundamental frequency for the comb filter: ";
   cin  >> fundFreq;
   comb.setFund(fundFreq);

   double scale;
   cout << "What scale factor for reverb [0..1] (1 rings forever): ";
   cin  >> scale;
   comb.setScale(scale);

   numSamples += 44100;
   Action action;
   action.tick(outsound, numSamples);

 
   return 0;
}



void exitUsage(char* commandName) {
   cout << endl;
   cout << 
      "A comb filter.  Uses whitenoise if no input sound is specified" << endl;
   cout << endl;
   cout << "Usage: " << commandName << " [insound] outsound" << endl;
   cout << endl;
   cout << "Program input: " << endl;
   cout << "   (a) amplitude of input noise (if no input sound)" << endl;
   cout << "   (b) duration of noise in samples (if no input sound)" << endl;
   cout << "   (c) fundamental frequency of comb filter in Hz" << endl;
   cout << "   (d) scale factor for feedback [0 .. 1]" << endl;
   cout << endl;

   exit(1);
}



