// combine soundfiles serially
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Thu May  1 11:57:59 GMT-0800 1997
// Last Modified: Sun May 25 01:51:04 GMT-0800 1997
// Filename:      .../sig/examples/concatenate/concatenate.cc
// Syntax:        C++ 
//

#include "Dependencies.h"

void exitUsage(char *commandName);


int main(int argc, char* argv[]) {
   if (argc < 3) exitUsage(argv[0]); 

   SoundHeader header;

   long numBlankSamples;
   cout << "How may blank samples between soundfiles: ";
   cin >> numBlankSamples;

   int numInSounds = argc - 2;

   SoundFileIn  insound;
   Constant     silence(0);

   int i;
   int chan = 1;
   for (i=0; i<numInSounds; i++) {
      header.setHeader(argv[i+1]);
      if (header.getChannels() == 2) {
         chan = 2;
         break;
      }
   }
   header.setChannels(chan);
   SoundFileOut outsound(argv[argc-1], header);

   Action action;

   for (i=0; i<numInSounds; i++) {
      insound.setFile(argv[i+1]);
      outsound.connect(insound, 0);
      if (header.getChannels() == 2) {
         if (insound.getChannels() == 2) {
            outsound.connect(insound, 1, 1);
         } else {  // duplicate mono input channel if a stereo output
            outsound.connect(insound, 1, 0);
         }
      }

      cerr << "Reading sound " << argv[i+1] << " which has " 
           << insound.getSamples() << " samples." << endl;

      action.tick(outsound, insound.getSamples());

      if (i < numInSounds-1) {
         outsound.connect(silence, 0);
         outsound.connect(silence, 0, 1);   
         action.tick(outsound, numBlankSamples);
      }
   }


   return 0;
}



void exitUsage(char* commandName) {
   cout << endl;
   cout << 
      "Strings together any number of soundfiles sequentially into one" << endl;
   cout << 
      "soundfile.  You can specify the number of zero samples between" << endl;
   cout << 
      "sounds.  If any input sound is a stereo file, then the output" << endl;
   cout <<
      "soundfile will be stereo, and any mono files will duplicate " << endl;
   cout <<
      "their channels into both the left and right channels of the " << endl;
   cout << "output sound file." << endl;
   cout << endl;
   cout << 
      "Usage: " << commandName << " insound1 [insound2 ...] outsound" << endl;
   cout << endl;
   cout << "Program input: " << endl;
   cout << "   (a) number of zero samples between input soundfiles." << endl;
   cout << endl;

   exit(1);
}



