// a resonance filter example.  If no input soundfile, then use whitenoise
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Wed May 14 23:44:12 GMT-0800 1997
// Last Modified: Sun May 25 01:00:43 GMT-0800 1997
// Filename:      .../sig/examples/resonator/resonator.cc
// Syntax:        C++ 
//

#include "Dependencies.h"

void exitUsage(char *commandName);

int main(int argc, char* argv[]) {
   if (argc == 1) exitUsage(argv[0]);

   SoundHeader header;
   char* infilename = "";
   char* outfilename = NULL;
   float noiseAmp = 0.0;
   long numSamples = 0;

   if (argc == 4) {            // no input sound
      header.setHighMono();
      outfilename = argv[3];
      cout << "What amplitude for the noise: [0 .. 1] ";
      cin >> noiseAmp;
      cout << "How many seconds of noise to use: ";
      float temp;
      cin >> temp;
      numSamples = (long)(temp * 44100);
   } else if (argc == 5) {     // input file specified
      header.setHeader(argv[3]);
      numSamples = header.getSamples() + 5;
      outfilename = argv[4];
      infilename = argv[3];
   } else {
      exitUsage(argv[0]); 
   }


   // Elements:
   SoundFileIn  insound(infilename);
   SoundFileOut outsound(outfilename, header);
   Resonator    resonator;
   WhiteNoise   noise(noiseAmp);
   Envelope     frequencyEnv(argv[1], numSamples);
   Envelope     bandwidthEnv(argv[2], numSamples);

   // Connections:
   outsound.connect(resonator);
   if (strlen(infilename) == 0) {         // use noise
      resonator.connect(noise, 0);      
   } else {                               // use input file
      resonator.connect(insound, 0);
   }
   resonator.connect(frequencyEnv, 1);
   resonator.connect(bandwidthEnv, 2);

   Action action;
   action.tick(outsound, numSamples);

   return 0;
}



void exitUsage(char* commandName) {
   cout << endl;
   cout << 
      "A frequency resonator.  Uses whitenoise if no input soundfile." << endl;
   cout << endl;
   cout << "Usage: " << commandName << " freqEnv bandwidthEnv";
   cout << " [insound] outsound" << endl;
   cout << endl;
   cout << "Program input: " << endl;
   cout << "   (a) amplitude scaling of file 1" << endl;
   cout << "   (b) amplitude scaling of file 2" << endl;
   cout << "   (c) offset time in seconds of second sound file" << endl;
   cout << endl;
   cout << "Example freq strings: pairs of time, Hz points" << endl;
   cout << "   constant env of 440: \"(0 440; 100 440)\"  " << endl;
   cout << "   triangle env:        \"(0 0; 1 440; 2 0)\" " << endl;
   cout << endl;

   exit(1);
}



