//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Fri Nov  1 23:35:02 PST 1996
// Last Modified: Mon Apr 28 21:52:13 GMT-0800 1997
// Filename:      ...sig/src/Generator/SoundFileIn/SoundFileIn.h
// Syntax:        C++ 
// $Smake:        cc -Wall -g -c %b.cc -I../../../include && rm -f %b.o
//

#ifndef _SOUNDFILEIN_H_INCLUDED
#define _SOUNDFILEIN_H_INCLUDED

#include <fstream.h>
#include <iostream.h>
#include <assert.h>

#include "Generator.h"
#include "SoundHeader.h"
#include "HelperFunctions.h"
#include "Array.h"
#include "FileIO.h"


class SoundFileIn : public Generator {
   public:
                        SoundFileIn           (char* aFileName = "",
                                                 float start=NONE,
                                                 float dur = NONE);
                       ~SoundFileIn();

      void              action                (void);
      int               getChannels           (void);
      long              getSamples            (void);
      SoundHeader&      getHeader             (void);
      sampleType        output                (int channel = 0);
      void              printState            (void);
      void              reset                 (int sampleIndex = 0);
      void              setFile               (char* aFileName, float start = 
                                                 NONE, float dur = NONE);

   protected:
      FileIO            sndfile;              // soundfile input stream
      char             *filename;             // name of the soundfile
      SoundHeader       sndHeader;            // header from soundfile
      long              curSampleIndex;       // the current sample number
      long              begSampleIndex;       // the first sample number
      long              endSampleIndex;       // the last sample number
      long              maxSampleIndex;       // the highest sample index value 
      int               loopingQ;
      int               runningQ;
      Array<sampleType> Output;               // output samples, by channel


      void              determineSampleReadingFunction   (void);
      void              (SoundFileIn::*fillOutputArray)  (void);

      // big endian samples
      void              fillOutputArray8M_B              (void); 
      void              fillOutputArray8L_B              (void); 
      void              fillOutputArray16L_B             (void); 
      void              fillOutputArray24L_B             (void); 
      void              fillOutputArray32L_B             (void); 
      void              fillOutputArray32F_B             (void); 
      void              fillOutputArray64F_B             (void); 

      // little endian samples
      void              fillOutputArray8M_L              (void); 
      void              fillOutputArray8L_L              (void); 
      void              fillOutputArray16L_L             (void); 
      void              fillOutputArray24L_L             (void); 
      void              fillOutputArray32L_L             (void); 
      void              fillOutputArray32F_L             (void); 
      void              fillOutputArray64F_L             (void); 

      short             mulaw8ToLinear16                 (uchar index);
};


#endif  _SOUNDFILEIN_H_INCLUDED



