//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sun Apr 20 21:01:10 GMT-0800 1997
// Last Modified: Sun Apr 20 21:01:13 GMT-0800 1997
// Filename:      .../sig/src/Filter/Allpass/Allpass.cc
// Syntax:        C++ 
// $Smake:        cc -Wall -g -c %b.cc -I../../../include && rm -f %b.o
//

#include <stdlib.h>
#include <math.h>
#include "Allpass.h"


//////////////////////////////
//
// Allpass::Allpass
//

Allpass::Allpass(void) {
   setName("Allpass");
   brandname = FILTER;
   outputValue = 0;
   setMaxDelay(4000);
   setDelay(1000);
   setGain(0.95);
}



//////////////////////////////
//
// Allpass::~Allpass
//

Allpass::~Allpass() { }



//////////////////////////////
//
// Allpass::action
//

void Allpass::action(void) {
   delayline.write(getSignal() + getGain() * delayline[-getDelay()]);
   outputValue = delayline[0] * -getGain() + delayline[-getDelay()+1];
}



//////////////////////////////
//
// Allpass::getDelay
//

sampleType Allpass::getDelay(void) {
   return inputs[1];
}



//////////////////////////////
//
// Allpass::getLoopTime
//

sampleType Allpass::getLoopTime(void) {
   return 1.0/getDelay();
}



//////////////////////////////
//
// Allpass::getMaxDelay
//

sampleType Allpass::getMaxDelay(void) {
   return delayline.getSize() - 1;
}



//////////////////////////////
//
// Allpass::getGain
//

sampleType Allpass::getGain(void) {
   return inputs[2];
}



//////////////////////////////
//
// Allpass::getSignal
//

sampleType Allpass::getSignal(void) {
   return inputs[0];
}



//////////////////////////////
//
// Allpass::output
//

sampleType Allpass::output(int channel) {
   return outputValue;
}



//////////////////////////////
//
// Allpass::setDelay
//

void Allpass::setDelay(sampleType aDelay) {
   connect(aDelay, 1);
}



//////////////////////////////
//
// Allpass::setLoopTime
//

void Allpass::setLoopTime(sampleType aLoopTime) {
   connect(1.0/aLoopTime, 1);
}



//////////////////////////////
//
// Allpass::setMaxDelay
//

void Allpass::setMaxDelay(long aMax) {
   delayline.setSize(aMax);
}



//////////////////////////////
//
// Allpass::setReverbTime
//

void Allpass::setReverbTime(sampleType aReverbTime) {
   setGain(pow(0.001, getLoopTime()/aReverbTime));
}



//////////////////////////////
//
// Allpass::setGain
//

void Allpass::setGain(sampleType aGain) {
   connect(aGain, 2);
}


